import { deepCopy, GridControlProps, GridControlState, IActionParam, MainControl } from '@ibiz-core';

/**
 * @description 表格部件
 * @export
 * @class GridControl
 * @extends {MainControl}
 */
export class GridControl extends MainControl {
  
  /**
   * @description 部件状态
   * @type {GridControlState}
   * @memberof GridControl
   */
  public declare controlState: GridControlState;

  /**
   * @description
   * @param {GridControlProps} props
   * @memberof GridControl
   */
  public setState(props: GridControlProps) {
    super.setState(props);
    this.controlState.multiple = toRef(props, 'multiple') as any;
    this.controlState.rowEditState = toRef(props, 'rowEditState') as any;
    this.controlState.rowActiveMode = toRef(props, 'rowActiveMode') as any;
    this.controlState.selectedData = toRef(props, 'selectedData') as any;
    this.controlState.selectFirstDefault = toRef(props, 'selectFirstDefault') as any;
  }

  /**
   * @description 表格数据改变
   * @param {number} rowIndex 行索引
   * @param {string} name 表格列属性名称
   * @param {*} value 表格列属性值
   * @memberof GridControl
   */
  public gridDataChange(rowIndex: number, name: string, value: any){
    this.controlState.items[rowIndex][name] = value;
  }

  /**
   * @description 使用加载功能模块
   * @param {GridControlProps} props 传入的props
   * @return {*} 
   * @memberof GridControl
   */
  public useLoad(props: GridControlProps){
    const { viewSubject, controlName } = this.controlState;
    const load = async (opt: any = {})=>{
      try {
        const loadAction = this.controlState.controlAction.loadAction;
        const { controlService, context, viewParams, showBusyIndicator } = this.controlState;
        const dataRef = toRef(this.controlState, 'items');
        
        if(!loadAction){
          return;
        }
        let _context = deepCopy(context);
        let _viewParams = deepCopy(viewParams);
    
        const response = await controlService.get(loadAction, _context, {viewParams: _viewParams}, showBusyIndicator );
        if (!response.status || response.status !== 200) {
          return
        }
        dataRef.value = response.data;
      } catch (error) {
        // todo 错误异常处理
      }
      
    }

    // 订阅viewSubject,监听load行为
    if(viewSubject){
      let subscription = viewSubject.subscribe(({ tag, action, data }: IActionParam)=>{
        if(Object.is(controlName, tag) && Object.is("load", action) ){
          load(data)
        }
      })
      // 部件卸载时退订viewSubject
      onUnmounted(()=>{
        subscription.unsubscribe();
      })
    }

    return {
      load: load
    }
  }

  /**
   * @description 处理编辑器事件
   * @param {IActionParam} actionParam 行为参数
   * @memberof GridControl
   */
  public handleEditorEvent(rowIndex: number, actionParam: IActionParam) {
    const { tag, action, data } = actionParam;
    switch (action) {
      case "valueChange":
        this.gridDataChange(rowIndex, tag, data);
        break;
      default:
        break;
    }
  }

  /**
   * @description 安装部件所有功能模块的方法
   * @param {GridControlProps} props 传入的Props
   * @param {Function} [emit]
   * @return {*} 
   * @memberof GridControl [emit] 事件
   */
  public moduleInstall(props: GridControlProps, emit?: Function) {
    const superParams = super.moduleInstall(props, emit);
    const { load } = this.useLoad(props)

    const handleEditorEvent = this.handleEditorEvent.bind(this);
    this.handleEditorEvent = (rowIndex: number,actionParam: IActionParam) => {
      handleEditorEvent(rowIndex, actionParam)
    }

    return {
      ...superParams,
      state: this.controlState,
      load,
      handleEditorEvent: this.handleEditorEvent,
    };
  }
}
